@@ -5,6 +5,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import shap
+import xgboost
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.model_selection import StratifiedKFold
 
@@ -56,10 +57,11 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         ``XGBClassifier``, ``RandomForestClassifier``,``RandomForestRegressor``,
         ``CatBoostClassifier``,``CatBoostRegressor``,
         ``BalancedRandomForestClassifier``,
-        ``LGBMClassifier``, and ``LGBMRegressor``.
+        ``LGBMClassifier``,  ``LGBMRegressor``, ``XGBSEKaplanNeighbors``,``XGBSEDebiasedBCE``
+        ``XGBSEBootstrapEstimator``
     estimator_params: dict
         Parameters passed to find the best estimator using optimization
-        method. For CATBOOST_CLASSIFICATION_PARAMS_DEFAULT are :     "nan_mode": "Min",
+        method. For CatBoostClassifier are :     "nan_mode": "Min",
         "eval_metric","iterations","sampling_frequency","leaf_estimation_method",
         "grow_policy","penalties_coefficient","boosting_type","model_shrink_mode",
         "feature_border_type","bayesian_matrix_reg","force_unit_auto_pair_weights",
@@ -71,7 +73,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         "min_data_in_leaf","loss_function","learning_rate","score_function",
         "task_type","leaf_estimation_iterations","bootstrap_type","max_leaves"
 
-        For CATBOOST_REGRESSION_PARAMS_DEFAULT are :
+        For CatBoostRegressor are :
         "nan_mode","eval_metric","iterations","sampling_frequency","leaf_estimation_method",
         "grow_policy","penalties_coefficient","boosting_type","model_shrink_mode",
         "feature_border_type","bayesian_matrix_reg","force_unit_auto_pair_weights",
@@ -83,7 +85,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         "loss_function","learning_rate","score_function","task_type",
         "leaf_estimation_iterations","bootstrap_type","max_leaves"
 
-        For XGBOOST_CLASSIFICATION_PARAMS_DEFAULT are :
+        For XGBClassifier are :
         "objective","use_label_encoder","base_score","booster",
         "callbacks","colsample_bylevel","colsample_bynode","colsample_bytree",
         "early_stopping_rounds","enable_categorical","eval_metric","gamma",
@@ -94,7 +96,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         "reg_alpha","reg_lambda","sampling_method","scale_pos_weight","subsample",
         "tree_method","validate_parameters","verbosity"
 
-        For XGBOOST_REGRESSION_PARAMS_DEFAULT are :
+        For XGBRegressor are :
         "objective","base_score","booster","callbacks","colsample_bylevel","colsample_bynode",
         "colsample_bytree","early_stopping_rounds","enable_categorical","eval_metric",
         "gamma","gpu_id","grow_policy","importance_type","interaction_constraints",
@@ -104,38 +106,64 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         "sampling_method","scale_pos_weight","subsample","tree_method","validate_parameters",
         "verbosity"
 
-        For RANDOMFOREST_CLASSIFICATION_PARAMS_DEFAULT are :
+        For RandomForestClassifier are :
         "n_estimators","criterion","max_depth","min_samples_split",
         "min_samples_leaf","min_weight_fraction_leaf","max_features",
         "max_leaf_nodes","min_impurity_decrease","bootstrap","oob_score",
         "n_jobs","random_state","verbose","warm_start","class_weight",
         "ccp_alpha","max_samples"
 
-        For RANDOMFOREST_REGRESSION_PARAMS_DEFAULT are :
+        For RandomForestRegressor are :
         "n_estimators","criterion","max_depth","min_samples_split",
         "min_samples_leaf","min_weight_fraction_leaf","max_features",
         "max_leaf_nodes","min_impurity_decrease","bootstrap","oob_score",
         "n_jobs","random_state","verbose","warm_start","ccp_alpha","max_samples"
 
-        For BLF_CLASSIFICATION_PARAMS_DEFAULT are :
+        For BalancedRandomForestClassifier are :
         "n_estimators","criterion"","max_depth","min_samples_split","min_samples_leaf",
         "min_weight_fraction_leaf","max_features","max_leaf_nodes","min_impurity_decrease",
         "bootstrap","oob_score","sampling_strategy","replacement","n_jobs","random_state",
         "verbose","warm_start","class_weight","ccp_alpha","max_samples"
 
-        For LGB_CLASSIFICATION_PARAMS_DEFAULT are:
+        For LGBMClassifier are:
         "boosting_type","num_leaves","max_depth","learning_rate","n_estimators",
         "subsample_for_bin","objective","class_weight","min_split_gain","min_child_weight",
         "min_child_samples","subsample","subsample_freq","colsample_bytree","reg_alpha",
         "reg_lambda","random_state","n_jobs","silent","importance_type"
 
-        For LGB_REGRESSION_PARAMS_DEFAULT are:
+        For LGBMRegressor are:
         "boosting_type","num_leaves","max_depth","learning_rate",
         "n_estimators","subsample_for_bin","objective","class_weight",
         "min_split_gain","min_child_weight","min_child_samples","subsample",
         "subsample_freq","colsample_bytree","reg_alpha","reg_lambda","random_state",
         "n_jobs","silent","importance_type"
 
+        For "XGBSEKaplanNeighbors","XGBSEDebiasedBCE", and "XGBSEBootstrapEstimator" if "duration" set
+        for "xgbse_focus" parameters are exactly like XGBMRegressor:
+        "objective","base_score","booster","callbacks","colsample_bylevel","colsample_bynode",
+        "colsample_bytree","early_stopping_rounds","enable_categorical","eval_metric",
+        "gamma","gpu_id","grow_policy","importance_type","interaction_constraints",
+        "learning_rate","max_bin","max_cat_to_onehot","max_delta_step","max_depth",
+        "max_leaves","min_child_weight","missing","monotone_constraints","n_estimators",
+        "n_jobs","num_parallel_tree","predictor","random_state","reg_alpha","reg_lambda",
+        "sampling_method","scale_pos_weight","subsample","tree_method","validate_parameters",
+        "verbosity"
+
+        For "XGBSEKaplanNeighbors","XGBSEDebiasedBCE", and "XGBSEBootstrapEstimator" if "event" set
+        for "xgbse_focus" parameters are exactly like XGBMClassifier:
+        "objective","use_label_encoder","base_score","booster",
+        "callbacks","colsample_bylevel","colsample_bynode","colsample_bytree",
+        "early_stopping_rounds","enable_categorical","eval_metric","gamma",
+        "gpu_id","grow_policy","importance_type","interaction_constraints",
+        "learning_rate","max_bin","max_cat_to_onehot","max_delta_step",
+        "max_depth","max_leaves","min_child_weight","missing","monotone_constraints",
+        "n_estimators","n_jobs","num_parallel_tree","predictor","random_state",
+        "reg_alpha","reg_lambda","sampling_method","scale_pos_weight","subsample",
+        "tree_method","validate_parameters","verbosity"
+
+        For a complete list of supported parameters, always check:
+        https://github.com/drhosseinjavedani/zoish/blob/main/zoish/model_conf.py
+
         https://shap-lrjball.readthedocs.io/en/docs_update/generated/shap.TreeExplainer.html
 
     model_output : str
@@ -237,6 +265,17 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
     cv : int, cross-validation generator or an iterable, default=None
         For more information visit
         https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
+    n_iter : int
+        Number of parameter settings that are sampled. n_iter trades off runtime vs quality of the solution.
+        For more information visit
+        https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
+    xgbse_focus : str
+        It is only applicable for xgbse (xgboost-survival-embeddings) models. If
+        the focus of feature selection is to optimize "duration" it should be set
+        xgbse_focus = "duration", if feature selection is to optimize "event" it should be set
+        xgbse_focus = "event". In both cases estimator argument, should be one of supported models,
+        i.e., "XGBSEKaplanNeighbors", "XGBSEDebiasedBCE", or "XGBSEBootstrapEstimator".
+
 
     Methods
     -------
@@ -289,39 +328,41 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         cv=StratifiedKFold(n_splits=3, shuffle=True),
         performance_metric="f1",
         n_iter=10,
+        xgbse_focus=None,
     ):
         """
-            Parameters
-            ----------
-        verbose : int
-            Controls the verbosity across all objects: the higher, the more messages.
-        random_state : int
-            Random number seed.
-        logging_basicConfig : object
-            Setting Logging process. Visit https://docs.python.org/3/library/logging.html
-        n_features : int
-            The number of features seen during:term:`fit`. Only defined if the
-            underlying estimator exposes such an attribute when fitted.
-        list_of_obligatory_features_that_must_be_in_model : [str]
-            A list of strings (columns names of feature set pandas data frame)
-            that should be among selected features. No matter if they have high or
-            low shap values, they will be selected at the end of the feature selection
-            step.
-        list_of_features_to_drop_before_any_selection :  [str]
-            A list of strings (columns names of feature set pandas data frame)
-            you want to exclude should be dropped before the selection process starts features.
-            For example, it is a good idea to exclude ``id`` and ``targets`` or ``class labels. ``
-            from feature space before selection starts.
+                Parameters
+                ----------
+            verbose : int
+                Controls the verbosity across all objects: the higher, the more messages.
+            random_state : int
+                Random number seed.
+            logging_basicConfig : object
+                Setting Logging process. Visit https://docs.python.org/3/library/logging.html
+            n_features : int
+                The number of features seen during:term:`fit`. Only defined if the
+                underlying estimator exposes such an attribute when fitted.
+            list_of_obligatory_features_that_must_be_in_model : [str]
+                A list of strings (columns names of feature set pandas data frame)
+                that should be among selected features. No matter if they have high or
+                low shap values, they will be selected at the end of the feature selection
+                step.
+            list_of_features_to_drop_before_any_selection :  [str]
+                A list of strings (columns names of feature set pandas data frame)
+                you want to exclude should be dropped before the selection process starts features.
+                For example, it is a good idea to exclude ``id`` and ``targets`` or ``class labels. ``
+                from feature space before selection starts.
         estimator: object
             An unfitted estimator. For now, only tree-based estimators. Supported
             methods are, "XGBRegressor",
             ``XGBClassifier``, ``RandomForestClassifier``,``RandomForestRegressor``,
             ``CatBoostClassifier``,``CatBoostRegressor``,
             ``BalancedRandomForestClassifier``,
-            ``LGBMClassifier``, and ``LGBMRegressor``.
+            ``LGBMClassifier``,  ``LGBMRegressor``, ``XGBSEKaplanNeighbors``,``XGBSEDebiasedBCE``
+            ``XGBSEBootstrapEstimator``
         estimator_params: dict
             Parameters passed to find the best estimator using optimization
-            method. For CATBOOST_CLASSIFICATION_PARAMS_DEFAULT are :     "nan_mode": "Min",
+            method. For CatBoostClassifier are :     "nan_mode": "Min",
             "eval_metric","iterations","sampling_frequency","leaf_estimation_method",
             "grow_policy","penalties_coefficient","boosting_type","model_shrink_mode",
             "feature_border_type","bayesian_matrix_reg","force_unit_auto_pair_weights",
@@ -333,7 +374,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
             "min_data_in_leaf","loss_function","learning_rate","score_function",
             "task_type","leaf_estimation_iterations","bootstrap_type","max_leaves"
 
-            For CATBOOST_REGRESSION_PARAMS_DEFAULT are :
+            For CatBoostRegressor are :
             "nan_mode","eval_metric","iterations","sampling_frequency","leaf_estimation_method",
             "grow_policy","penalties_coefficient","boosting_type","model_shrink_mode",
             "feature_border_type","bayesian_matrix_reg","force_unit_auto_pair_weights",
@@ -345,7 +386,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
             "loss_function","learning_rate","score_function","task_type",
             "leaf_estimation_iterations","bootstrap_type","max_leaves"
 
-            For XGBOOST_CLASSIFICATION_PARAMS_DEFAULT are :
+            For XGBClassifier are :
             "objective","use_label_encoder","base_score","booster",
             "callbacks","colsample_bylevel","colsample_bynode","colsample_bytree",
             "early_stopping_rounds","enable_categorical","eval_metric","gamma",
@@ -356,7 +397,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
             "reg_alpha","reg_lambda","sampling_method","scale_pos_weight","subsample",
             "tree_method","validate_parameters","verbosity"
 
-            For XGBOOST_REGRESSION_PARAMS_DEFAULT are :
+            For XGBRegressor are :
             "objective","base_score","booster","callbacks","colsample_bylevel","colsample_bynode",
             "colsample_bytree","early_stopping_rounds","enable_categorical","eval_metric",
             "gamma","gpu_id","grow_policy","importance_type","interaction_constraints",
@@ -366,145 +407,178 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
             "sampling_method","scale_pos_weight","subsample","tree_method","validate_parameters",
             "verbosity"
 
-            For RANDOMFOREST_CLASSIFICATION_PARAMS_DEFAULT are :
+            For RandomForestClassifier are :
             "n_estimators","criterion","max_depth","min_samples_split",
             "min_samples_leaf","min_weight_fraction_leaf","max_features",
             "max_leaf_nodes","min_impurity_decrease","bootstrap","oob_score",
             "n_jobs","random_state","verbose","warm_start","class_weight",
             "ccp_alpha","max_samples"
 
-            For RANDOMFOREST_REGRESSION_PARAMS_DEFAULT are :
+            For RandomForestRegressor are :
             "n_estimators","criterion","max_depth","min_samples_split",
             "min_samples_leaf","min_weight_fraction_leaf","max_features",
             "max_leaf_nodes","min_impurity_decrease","bootstrap","oob_score",
             "n_jobs","random_state","verbose","warm_start","ccp_alpha","max_samples"
 
-            For BLF_CLASSIFICATION_PARAMS_DEFAULT are :
+            For BalancedRandomForestClassifier are :
             "n_estimators","criterion"","max_depth","min_samples_split","min_samples_leaf",
             "min_weight_fraction_leaf","max_features","max_leaf_nodes","min_impurity_decrease",
             "bootstrap","oob_score","sampling_strategy","replacement","n_jobs","random_state",
             "verbose","warm_start","class_weight","ccp_alpha","max_samples"
 
-            For LGB_CLASSIFICATION_PARAMS_DEFAULT are:
+            For LGBMClassifier are:
             "boosting_type","num_leaves","max_depth","learning_rate","n_estimators",
             "subsample_for_bin","objective","class_weight","min_split_gain","min_child_weight",
             "min_child_samples","subsample","subsample_freq","colsample_bytree","reg_alpha",
             "reg_lambda","random_state","n_jobs","silent","importance_type"
 
-            For LGB_REGRESSION_PARAMS_DEFAULT are:
+            For LGBMRegressor are:
             "boosting_type","num_leaves","max_depth","learning_rate",
             "n_estimators","subsample_for_bin","objective","class_weight",
             "min_split_gain","min_child_weight","min_child_samples","subsample",
             "subsample_freq","colsample_bytree","reg_alpha","reg_lambda","random_state",
             "n_jobs","silent","importance_type"
 
+            For "XGBSEKaplanNeighbors","XGBSEDebiasedBCE", and "XGBSEBootstrapEstimator" if "duration" set
+            for "xgbse_focus" parameters are exactly like XGBMRegressor:
+            "objective","base_score","booster","callbacks","colsample_bylevel","colsample_bynode",
+            "colsample_bytree","early_stopping_rounds","enable_categorical","eval_metric",
+            "gamma","gpu_id","grow_policy","importance_type","interaction_constraints",
+            "learning_rate","max_bin","max_cat_to_onehot","max_delta_step","max_depth",
+            "max_leaves","min_child_weight","missing","monotone_constraints","n_estimators",
+            "n_jobs","num_parallel_tree","predictor","random_state","reg_alpha","reg_lambda",
+            "sampling_method","scale_pos_weight","subsample","tree_method","validate_parameters",
+            "verbosity"
+
+            For "XGBSEKaplanNeighbors","XGBSEDebiasedBCE", and "XGBSEBootstrapEstimator" if "event" set
+            for "xgbse_focus" parameters are exactly like XGBMClassifier:
+            "objective","use_label_encoder","base_score","booster",
+            "callbacks","colsample_bylevel","colsample_bynode","colsample_bytree",
+            "early_stopping_rounds","enable_categorical","eval_metric","gamma",
+            "gpu_id","grow_policy","importance_type","interaction_constraints",
+            "learning_rate","max_bin","max_cat_to_onehot","max_delta_step",
+            "max_depth","max_leaves","min_child_weight","missing","monotone_constraints",
+            "n_estimators","n_jobs","num_parallel_tree","predictor","random_state",
+            "reg_alpha","reg_lambda","sampling_method","scale_pos_weight","subsample",
+            "tree_method","validate_parameters","verbosity"
+
+            For a complete list of supported parameters, always check:
+            https://github.com/drhosseinjavedani/zoish/blob/main/zoish/model_conf.py
+
             https://shap-lrjball.readthedocs.io/en/docs_update/generated/shap.TreeExplainer.html
+            model_output : str
+                    "raw", "probability", "log_loss", or model method name
+                    What output of the model should be explained. If "raw" then we explain the raw output of the
+                    trees, which varies by model. For regression models "raw" is the standard output, for binary
+                    classification in XGBoost this is the log odds ratio. If model_output is the name of a supported
+                    prediction method on the model object then we explain the output of that model method name.
+                    For example model_output="predict_proba" explains the result of calling model.predict_proba.
+                    If "probability" then we explain the output of the model transformed into probability space
+                    (note that this means the SHAP values now sum to the probability output of the model). If "logloss"
+                    then we explain the log base e of the model loss function, so that the SHAP values sum up to the
+                    log loss of the model for each sample. This is helpful for breaking down model performance by feature.
+                    Currently the probability and logloss options are only supported when feature_dependence="independent".
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
 
-        model_output : str
-                "raw", "probability", "log_loss", or model method name
-                What output of the model should be explained. If "raw" then we explain the raw output of the
-                trees, which varies by model. For regression models "raw" is the standard output, for binary
-                classification in XGBoost this is the log odds ratio. If model_output is the name of a supported
-                prediction method on the model object then we explain the output of that model method name.
-                For example model_output="predict_proba" explains the result of calling model.predict_proba.
-                If "probability" then we explain the output of the model transformed into probability space
-                (note that this means the SHAP values now sum to the probability output of the model). If "logloss"
-                then we explain the log base e of the model loss function, so that the SHAP values sum up to the
-                log loss of the model for each sample. This is helpful for breaking down model performance by feature.
-                Currently the probability and logloss options are only supported when feature_dependence="independent".
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            feature_perturbation : str
+                    "interventional" (default) or "tree_path_dependent" (default when data=None)
+                    Since SHAP values rely on conditional expectations we need to decide how to handle correlated
+                    (or otherwise dependent) input features. The "interventional" approach breaks the dependencies between
+                    features according to the rules dictated by causal inference (Janzing et al. 2019). Note that the
+                    "interventional" option requires a background dataset and its runtime scales linearly with the size
+                    of the background dataset you use. Anywhere from 100 to 1000 random background samples are good
+                    sizes to use. The "tree_path_dependent" approach is to just follow the trees and use the number
+                    of training examples that went down each leaf to represent the background distribution. This approach
+                    does not require a background dataset and so is used by default when no background dataset is provided.
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
 
-        feature_perturbation : str
-                "interventional" (default) or "tree_path_dependent" (default when data=None)
-                Since SHAP values rely on conditional expectations we need to decide how to handle correlated
-                (or otherwise dependent) input features. The "interventional" approach breaks the dependencies between
-                features according to the rules dictated by causal inference (Janzing et al. 2019). Note that the
-                "interventional" option requires a background dataset and its runtime scales linearly with the size
-                of the background dataset you use. Anywhere from 100 to 1000 random background samples are good
-                sizes to use. The "tree_path_dependent" approach is to just follow the trees and use the number
-                of training examples that went down each leaf to represent the background distribution. This approach
-                does not require a background dataset and so is used by default when no background dataset is provided.
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            algorithm : str
+                    "auto" (default), "v0", "v1" or "v2"
+                    The "v0" algorithm refers to TreeSHAP algorithm in SHAP package (https://github.com/slundberg/shap).
+                    The "v1" and "v2" algorithms refer to Fast TreeSHAP v1 algorithm and Fast TreeSHAP v2 algorithm
+                    proposed in paper https://arxiv.org/abs/2109.09847 (Jilei 2021). In practice, Fast TreeSHAP v1 is 1.5x
+                    faster than TreeSHAP while keeping the memory cost unchanged, and Fast TreeSHAP v2 is 2.5x faster than
+                    TreeSHAP at the cost of a slightly higher memory usage. The default value of algorithm is "auto",
+                    which automatically chooses the most appropriate algorithm to use. Specifically, we always prefer
+                    "v1" over "v0", and we prefer "v2" over "v1" when the number of samples to be explained is sufficiently
+                    large, and the memory constraint is also satisfied.
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
 
-        algorithm : str
-                "auto" (default), "v0", "v1" or "v2"
-                The "v0" algorithm refers to TreeSHAP algorithm in SHAP package (https://github.com/slundberg/shap).
-                The "v1" and "v2" algorithms refer to Fast TreeSHAP v1 algorithm and Fast TreeSHAP v2 algorithm
-                proposed in paper https://arxiv.org/abs/2109.09847 (Jilei 2021). In practice, Fast TreeSHAP v1 is 1.5x
-                faster than TreeSHAP while keeping the memory cost unchanged, and Fast TreeSHAP v2 is 2.5x faster than
-                TreeSHAP at the cost of a slightly higher memory usage. The default value of algorithm is "auto",
-                which automatically chooses the most appropriate algorithm to use. Specifically, we always prefer
-                "v1" over "v0", and we prefer "v2" over "v1" when the number of samples to be explained is sufficiently
-                large, and the memory constraint is also satisfied.
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            shap_n_jobs : int
+                    (default), or a positive integer
+                    Number of parallel threads used to run Fast TreeSHAP. The default value of n_jobs is -1, which utilizes
+                    all available cores in parallel computing (Setting OMP_NUM_THREADS is unnecessary since n_jobs will
+                    overwrite this parameter).
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
 
-        shap_n_jobs : int
-                (default), or a positive integer
-                Number of parallel threads used to run Fast TreeSHAP. The default value of n_jobs is -1, which utilizes
-                all available cores in parallel computing (Setting OMP_NUM_THREADS is unnecessary since n_jobs will
-                overwrite this parameter).
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            random_search_n_jobs : int
+                    (default), or a positive integer
 
-        random_search_n_jobs : int
-                (default), or a positive integer
+            memory_tolerance : int
+                    (default), or a positive number
+                    Upper limit of memory allocation (in GB) to run Fast TreeSHAP v2. The default value of memory_tolerance is -1,
+                    which allocates a maximum of 0.25 * total memory of the machine to run Fast TreeSHAP v2.
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            feature_names : [str]
+                    Feature names.
+            approximate : bool
+                    Run fast, but only roughly approximate the Tree SHAP values. This runs a method
+                    previously proposed by Saabas which only considers a single feature ordering. Take care
+                    since this does not have the consistency guarantees of Shapley values and places too
+                    much weight on lower splits in the tree.
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            shortcut: False (default) or True
+                    Whether to use the C++ version of TreeSHAP embedded in XGBoost, LightGBM and CatBoost packages directly
+                    when computing SHAP values for XGBoost, LightGBM and CatBoost models, and when computing SHAP interaction
+                    values for XGBoost models. Current version of FastTreeSHAP package supports XGBoost and LightGBM models,
+                    and its support to CatBoost model is working in progress (shortcut is automatically set to be True for
+                    CatBoost model).
+                    For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
+            plot_shap_summary : bool
+                Set True if you want to see shap summary plot of
+                selected features. (default ``False``)
+            save_shap_summary_plot : bool
+                Set True if you want to save a shap summary plot with data_time
+                label root directory of your project. (default ``False``)
+            path_to_save_plot = str
+                Path for saving summary plot.
+            shap_fig = object
+                matplotlib pyplot figure object.
+                For more info visit : https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.figure.html
+            test_size : float or int
+                If float, it should be between 0.0 and 1.0 and represent the proportion
+                of the dataset to include in the train split during estimating the best estimator
+                by optimization method. If int represents the
+                absolute number of train samples. If None, the value is automatically
+                set to the complement of the test size.
+            performance_metric : str
+                Measurement of performance for classification and
+                regression estimator during hyperparameter optimization while
+                estimating best estimator. Classification-supported measurments are
+                f1, f1_score, acc, accuracy_score, pr, precision_score,
+                recall, recall_score, roc, roc_auc_score, roc_auc,
+                tp, true positive, tn, true negative. Regression supported
+                measurements are r2, r2_score, explained_variance_score,
+                max_error, mean_absolute_error, mean_squared_error,
+                median_absolute_error, and mean_absolute_percentage_error.
+            cv : int, cross-validation generator or an iterable, default=None
+                For more information visit
+                https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
+            n_iter : int
+                Number of parameter settings that are sampled. n_iter trades off runtime vs quality of the solution.
+                For more information visit
+                https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
+            xgbse_focus : str
+                It is only applicable for xgbse (xgboost-survival-embeddings) models. If
+                the focus of feature selection is to optimize "duration" it should be set
+                xgbse_focus = "duration", if feature selection is to optimize "event" it should be set
+                xgbse_focus = "event". In both cases estimator argument, should be one of supported models,
+                i.e., "XGBSEKaplanNeighbors", "XGBSEDebiasedBCE", or "XGBSEBootstrapEstimator".
 
-        memory_tolerance : int
-                (default), or a positive number
-                Upper limit of memory allocation (in GB) to run Fast TreeSHAP v2. The default value of memory_tolerance is -1,
-                which allocates a maximum of 0.25 * total memory of the machine to run Fast TreeSHAP v2.
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
-        feature_names : [str]
-                Feature names.
-        approximate : bool
-                Run fast, but only roughly approximate the Tree SHAP values. This runs a method
-                previously proposed by Saabas which only considers a single feature ordering. Take care
-                since this does not have the consistency guarantees of Shapley values and places too
-                much weight on lower splits in the tree.
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
-        shortcut: False (default) or True
-                Whether to use the C++ version of TreeSHAP embedded in XGBoost, LightGBM and CatBoost packages directly
-                when computing SHAP values for XGBoost, LightGBM and CatBoost models, and when computing SHAP interaction
-                values for XGBoost models. Current version of FastTreeSHAP package supports XGBoost and LightGBM models,
-                and its support to CatBoost model is working in progress (shortcut is automatically set to be True for
-                CatBoost model).
-                For more info visit : https://github.com/linkedin/FastTreeSHAP/blob/master/fasttreeshap/explainers/_tree.py
-        plot_shap_summary : bool
-            Set True if you want to see shap summary plot of
-            selected features. (default ``False``)
-        save_shap_summary_plot : bool
-            Set True if you want to save a shap summary plot with data_time
-            label root directory of your project. (default ``False``)
-        path_to_save_plot = str
-            Path for saving summary plot.
-        shap_fig = object
-            matplotlib pyplot figure object.
-            For more info visit : https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.figure.html
-        test_size : float or int
-            If float, it should be between 0.0 and 1.0 and represent the proportion
-            of the dataset to include in the train split during estimating the best estimator
-            by optimization method. If int represents the
-            absolute number of train samples. If None, the value is automatically
-            set to the complement of the test size.
-        performance_metric : str
-            Measurement of performance for classification and
-            regression estimator during hyperparameter optimization while
-            estimating best estimator. Classification-supported measurments are
-            f1, f1_score, acc, accuracy_score, pr, precision_score,
-            recall, recall_score, roc, roc_auc_score, roc_auc,
-            tp, true positive, tn, true negative. Regression supported
-            measurements are r2, r2_score, explained_variance_score,
-            max_error, mean_absolute_error, mean_squared_error,
-            median_absolute_error, and mean_absolute_percentage_error.
-        cv : int, cross-validation generator or an iterable, default=None
-            For more information visit
-            https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
-        n_iter : int
-            Number of parameter settings that are sampled. n_iter trades off runtime vs quality of the solution.
-            For more information visit
-            https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html
         """
         self.logging_basicConfig = logging_basicConfig
         self.verbose = verbose
+        self.xgbse_focus = xgbse_focus
         self.random_state = random_state
         self.n_features = n_features
         self.list_of_obligatory_features_that_must_be_in_model = (
@@ -539,6 +613,16 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
         self.best_estimator = None
         self.importance_df = None
 
+    @property
+    def xgbse_focus(self):
+        logging.info("Getting value for xgbse_focus")
+        return self._xgbse_focus
+
+    @xgbse_focus.setter
+    def xgbse_focus(self, value):
+        logging.info("Setting value for xgbse_focus")
+        self._xgbse_focus = value
+
     @property
     def logging_basicConfig(self):
         logging.info("Getting value for logging_basicConfig")
@@ -629,6 +713,41 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
     @estimator_params.setter
     def estimator_params(self, value):
         logging.info(self.estimator)
+        # get parameters for xgbse.XGBSEKaplanNeighbors and others and check if
+        # the selected parameters in the list or not
+        if (
+            self.estimator.__class__.__name__ == "XGBSEKaplanNeighbors"
+            or self.estimator.__class__.__name__ == "XGBSEDebiasedBCE"
+            or self.estimator.__class__.__name__ == "XGBSEBootstrapEstimator"
+            and self.xgbse_focus == "duration"
+        ):
+
+            if value.keys() <= XGBOOST_REGRESSION_PARAMS_DEFAULT.keys():
+                logging.info("Setting value for estimator_params")
+                self._estimator_params = value
+            else:
+                raise TypeError(
+                    f"error occures during parameter checking for \
+                        {value.__class__.__name__}"
+                )
+            # change the base estimator to XGBRegressor
+            self.estimator = xgboost.XGBRegressor()
+        if (
+            self.estimator.__class__.__name__ == "XGBSEKaplanNeighbors"
+            or self.estimator.__class__.__name__ == "XGBSEDebiasedBCE"
+            or self.estimator.__class__.__name__ == "XGBSEBootstrapEstimator"
+            and self.xgbse_focus == "event"
+        ):
+            if value.keys() <= XGBOOST_CLASSIFICATION_PARAMS_DEFAULT.keys():
+                logging.info("Setting value for estimator_params")
+                self._estimator_params = value
+            else:
+                raise TypeError(
+                    f"error occures during parameter checking for \
+                        {value.__class__.__name__}"
+                )
+            # change the base estimator to XGBClassifier
+            self.estimator = xgboost.XGBClassifier()
         # get parameters for lightgbm.LGBMRegressor and check if
         # the selected parameters in the list or not
         if self.estimator.__class__.__name__ == "LGBMRegressor":
@@ -935,6 +1054,7 @@ class RandomizedSearchCVShapFeatureSelector(BaseEstimator, TransformerMixin):
             self.random_search_n_jobs,
             self.n_iter,
             self.cv,
+            self.xgbse_focus,
         )
         print(self.estimator.__class__.__name__)
 
